/* GCompris - algebra.js
 *
 * Copyright (C) 2014 Aruna Sankaranarayanan and Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <https://www.gnu.org/licenses/>.
 */

.pragma library
.import QtQuick 2.6 as Quick
.import "qrc:/gcompris/src/core/core.js" as Core

var currentLevel
var maxLevel
var coreItems
var otherItems
var dataset
var operand
var secondOperandVal
var firstOperandVal
var subLevelData = []
var speedSetting
var OperandsEnum = {
    TIMES_SIGN : "\u00D7",
    PLUS_SIGN : "\u002B",
    MINUS_SIGN : "\u2212",
    DIVIDE_SIGN : "\u2215"
}


function start(coreItems_, otherItems_, operand_, speedSetting_) {
    operand   = operand_
    coreItems = coreItems_
    otherItems = otherItems_
    speedSetting = speedSetting_
    currentLevel = 0
    dataset = coreItems.levels
    maxLevel = dataset.length
    initLevel()
}

function stop() {
    coreItems.balloon.stopMoving()
}

function initLevel() {
    coreItems.bar.level = currentLevel + 1
    coreItems.score.visible = false
    coreItems.okButton.visible = false
    coreItems.score.currentSubLevel = 1
    subLevelData = []
    if(currentLevel === maxLevel)
        initLastLevel();
    else {
        coreItems.score.numberOfSubLevels = dataset[currentLevel].operands.length;
        for(var i = 0; i < coreItems.score.numberOfSubLevels; i++)
            subLevelData.push([dataset[currentLevel].operands[i].first, dataset[currentLevel].operands[i].second])
    }
    subLevelData = Core.shuffle(subLevelData)
    calculateOperands()
    otherItems.iAmReady.visible = true
    otherItems.firstOp.visible = false
    otherItems.secondOp.visible = false
    coreItems.balloon.stopMoving()
}

function circularShiftElements() {
    if(!validateAnswer(parseInt(otherItems.numpad.answer)))
        subLevelData.push(subLevelData.shift())
    else
        subLevelData.shift()
}

function nextLevel() {
    if(maxLevel  < ++currentLevel) {
        currentLevel = 0
    }
    initLevel();
}

function previousLevel() {
    if(--currentLevel < 0) {
        currentLevel = maxLevel
    }
    initLevel();
}

/* last level will be generated by this function,it will always contain randomly picked questions from previous levels*/
function initLastLevel() {
    coreItems.score.numberOfSubLevels = maxLevel  * 5;
    for (var i = 0; i < coreItems.score.numberOfSubLevels; i++) {
        //randomly choose a level
        var randomLevel = Math.floor(Math.random() * maxLevel)
        // randomly choose question from level
        var randomIndex = Math.floor(Math.random() * dataset[randomLevel].operands.length)
        subLevelData.push([dataset[randomLevel].operands[randomIndex].first, dataset[randomLevel].operands[randomIndex].second])
    }
}

function calculateOperands()
{

        firstOperandVal = subLevelData[0][0]
        secondOperandVal = subLevelData[0][1]

    otherItems.firstOp.text = firstOperandVal
    otherItems.secondOp.text = secondOperandVal
}

// Return the expected answer
function getAnswer() {
    switch(operand.text)
    {
    case OperandsEnum.TIMES_SIGN:
        return (firstOperandVal * secondOperandVal)

    case OperandsEnum.PLUS_SIGN:
        return (firstOperandVal + secondOperandVal)

    case OperandsEnum.MINUS_SIGN:
        return (firstOperandVal - secondOperandVal)

    case OperandsEnum.DIVIDE_SIGN:
        return (firstOperandVal / secondOperandVal)
    }
}

function validateAnswer(screenAnswer)
{
    return (getAnswer() === screenAnswer)
}

function run() {
    calculateOperands()
    otherItems.numpad.resetText()
    coreItems.score.visible = true
    otherItems.iAmReady.visible = false
    otherItems.firstOp.visible = true
    otherItems.secondOp.visible = true
    coreItems.okButton.visible = true
    otherItems.numpad.answerFlag = false
    otherItems.result = getAnswer()
    coreItems.balloon.startMoving(100000 / speedSetting)
}

function questionsLeft() {
    coreItems.okButton.enabled = false
    coreItems.balloon.startMoving(100000 / speedSetting)
    circularShiftElements()
    if(validateAnswer(parseInt(otherItems.numpad.answer))) {
        otherItems.numpad.answerFlag = true

        if(coreItems.score.currentSubLevel < coreItems.score.numberOfSubLevels) {
            coreItems.audioEffects.play("qrc:/gcompris/src/core/resource/sounds/win.wav")
            coreItems.score.currentSubLevel++
            coreItems.timer.start()
        } else {
            coreItems.score.currentSubLevel = 1
            coreItems.balloon.stopMoving()
            coreItems.bonus.good("smiley");
        }
    }
    else
        coreItems.bonus.bad("smiley");
}
